-----------------------------------------------------------------------------

local AloftAlpha = Aloft:NewModule("Alpha", "AceEvent-2.0", "AceHook-2.1")
AloftAlpha.db = Aloft:AcquireDBNamespace("Alpha")
Aloft:RegisterDefaults("Alpha", "profile", {
	targetEnable		= false,
	targetAlpha			= 1.0,
	defaultEnable		= false,
	defaultAlpha		= 1.0,
	nonTargetEnable		= false,
	nonTargetAlpha		= 0.80,
})

-----------------------------------------------------------------------------

local updateFunc
local targetNameplate

function AloftAlpha:RegisterEvents()
	self:UnregisterAllEvents()
	self:UnhookAll()

	updateFunc = self:getUpdateFunc()
	
	if updateFunc then
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")

		-- Since Aloft requires alpha information to determine target nameplates, 
		-- and this module adjusts them, this module then needs to take over
		-- responsibility for determining the target nameplate
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:Hook(Aloft, "GetTargetNameplate", "GetTargetNameplate")
	
		for aloftData in Aloft:IterateVisibleNameplates() do
			self:HookScript(aloftData.nameplateFrame, "OnUpdate", "OnUpdateHook")
		end
	end
end

function AloftAlpha:EnableChange()
	self:RegisterEvents()
end

-----------------------------------------------------------------------------

function AloftAlpha:OnInitialize()
	profile = self.db.profile
end

function AloftAlpha:OnEnable()
	self:RegisterEvents()
end

function AloftAlpha:OnDisable()
end

-----------------------------------------------------------------------------

function AloftAlpha:GetTargetNameplate(this)
	-- Since Aloft requires alpha information to determine target nameplates, 
	-- and this module adjusts them, this module then needs to take over
	-- responsibility for determining the target nameplate
	return targetNameplate
end

function AloftAlpha:PLAYER_TARGET_CHANGED()
	targetNameplate = nil
end

-----------------------------------------------------------------------------

local function allEnabled(this)
	if this:GetAlpha() ~= 1 then		--non-target, target selected
		this:SetAlpha(profile.nonTargetAlpha)
	elseif UnitExists("target") then	--target, target selected
		targetNameplate = this.aloftData
		this:SetAlpha(profile.targetAlpha)
	else					--all units, no target selected
		this:SetAlpha(profile.defaultAlpha)
	end
end

local function allButNon(this)
	if not UnitExists("target") then --default
		this:SetAlpha(profile.defaultAlpha)
	elseif this:GetAlpha() == 1 then --target
		targetNameplate = this.aloftData
		this:SetAlpha(profile.targetAlpha)
	end
end

local function allButDefault(this)
	if this:GetAlpha() ~= 1 then
		this:SetAlpha(profile.nonTargetAlpha)
	elseif UnitExists("target") then
		targetNameplate = this.aloftData
		this:SetAlpha(profile.targetAlpha)
	end
end

local function targetOnly(this)
	if UnitExists("target") and this:GetAlpha() == 1 then
		targetNameplate = this.aloftData
		this:SetAlpha(profile.targetAlpha)
	end
end

local function allButTarget(this)
	if not UnitExists("target") then
		this:SetAlpha(profile.defaultAlpha)
	elseif this:GetAlpha() ~= 1 then
		this:SetAlpha(profile.nonTargetAlpha)
	else
		targetNameplate = this.aloftData
	end
end

local function defaultOnly(this)
	if not UnitExists("target") then
		this:SetAlpha(profile.defaultAlpha)
	elseif this:GetAlpha() == 1 then
		targetNameplate = this.aloftData
	end
end

local function nonOnly(this)
	if this:GetAlpha() ~= 1 then
		this:SetAlpha(profile.nonTargetAlpha)
	elseif UnitExists("target") then
		targetNameplate = this.aloftData
	end
end

function AloftAlpha:getUpdateFunc()
	if profile.targetEnable then
		if profile.defaultEnable then
			if profile.nonTargetEnable then
				return allEnabled
			else
				return allButNon
			end
		elseif profile.nonTargetEnable then
			return allButDefault
		else
			return targetOnly
		end
	elseif profile.defaultEnable then
		if profile.nonTargetEnable then
			return allButTarget
		else
			return defaultOnly
		end
	elseif profile.nonTargetEnable then
		return nonOnly
	end
	return nil
end

function AloftAlpha:OnUpdateHook(this)
	updateFunc(this)
	self.hooks[this]["OnUpdate"](this)
end

-----------------------------------------------------------------------------

function AloftAlpha:OnNameplateShow(aloftData)
	self:HookScript(aloftData.nameplateFrame, "OnUpdate", "OnUpdateHook")
end

function AloftAlpha:OnNameplateHide(aloftData)
	self:Unhook(aloftData.nameplateFrame, "OnUpdate")
	if targetNameplate == aloftData then
		targetNameplate = nil
	end
end

-----------------------------------------------------------------------------
